const frontMatter = require('front-matter');
const path = require('path');
const through = require('through2');
const _ = require('lodash');
const utils = require('./utils');

/**
 * Renders a page with a layout. The page also has access to any loaded partials, helpers, or data.
 * @param {object} file - Vinyl file being parsed.
 * @param {string} enc - Vinyl file encoding.
 * @param {function} cb - Callback that passes the rendered page through the stream.
 */
let layoutTemplate;

function render(file, enc, cb) {
    try {
        // Get the HTML for the current page and layout
        const page = frontMatter(file.contents.toString());

        // Determine which layout to use
        const basePath = path.dirname(file.path);
        const layout =
            page.attributes.layout ||
            (this.options.pageLayouts && this.options.pageLayouts[basePath]) ||
            'default';
        layoutTemplate = this.layouts[layout];

        if (!layoutTemplate) {
            if (layout === 'default') {
                throw new Error('Panini error: you must have a layout named "default".');
            } else {
                throw new Error(`Panini error: no layout named "${layout}" exists.`);
            }
        }

        // Now create Handlebars templates out of them
        const pageTemplate = this.Handlebars.compile(`${page.body}\n`);

        const pageName = path.basename(file.path, '.html');
        const dataFileName = `${pageName}.json`;
        const dataPath = path.join(basePath, dataFileName);

        let pageData = {};
        if (pageName !== 'index') {
            try {
                pageData = utils.loadJSON(dataPath);
            } catch (error) {
                console.log(error.message);
            }
        }

        // Add any data from stream plugins
        _.extend(pageData, file.data || {});

        // Add this page's front matter
        _.extend(pageData, page.attributes);

        // Finish by adding constants
        _.extend(pageData, {
            page: pageName,
            layout,
        });

        // Finally, add the page as a partial called "body", and render the layout template
        this.Handlebars.registerPartial('body', pageTemplate);
        file.contents = new Buffer(layoutTemplate(pageData));
    } catch (error) {
        if (layoutTemplate) {
            // Layout was parsed properly so we can insert the error message into the body of the
            // layout
            this.Handlebars.registerPartial('body',
                'Panini: template could not be parsed <br>\n <pre>{{ error }}</pre>');
            file.contents = new Buffer(layoutTemplate({ error }));
        } else {
            // Not even once - write error directly into the HTML output so the user gets an error
            // Maintain basic HTML structure to allow Livereloading scripts to be injected properly
            const errorHTML =
                `<!DOCTYPE html><html><head><title>Panini error</title></head><body><pre>
                ${error}
                </pre></body></html>`;
            file.contents = new Buffer(errorHTML);
        }

        // Log the error into console
        console.log('Panini: rendering error ocurred.\n', error);
    } finally {
        // This sends the modified file back into the stream
        cb(null, file);
    }
}

module.exports = function boundRender() {
    return through.obj(render.bind(this));
};
